package com.ibm.websphere.samples.bank.ejb;

import java.rmi.RemoteException;

import javax.ejb.EJBException;
import javax.ejb.FinderException;
import javax.ejb.ObjectNotFoundException;
import javax.naming.InitialContext;
import javax.naming.NamingException;

import com.ibm.websphere.samples.bank.exception.InsufficientFundsException;

/**
 * Bean implementation class for Enterprise Bean: Transfer
 */
public class TransferBean implements javax.ejb.SessionBean {
	private javax.ejb.SessionContext mySessionCtx;
	
	private AccountLocalHome accountHome = null;
	private final static String ACCOUNT_JNDI_NAME = "java:comp/env/ejb/Account";

	/**
	 * Returns the balance of the account identified by the acctId parameter.
	 * @return float The account balance.
	 * @param acctId long The account Id for which a balance is returned.
	 * @exception java.rmi.RemoteException A server-side exception caused the method to fail.
	 * @exception javax.ejb.FinderException The specified account Id was not found.
	 */
	public float getBalance(int acctId) throws FinderException, EJBException {
	    AccountKey key = new AccountKey(acctId);
	    AccountLocal fromAccount;
	    try {
	        fromAccount = accountHome.findByPrimaryKey(key);
	        return fromAccount.getBalance();
	    } catch (ObjectNotFoundException ex) {
	        throw new FinderException("Account " + acctId + " does not exist.");
	    } catch (FinderException ex) {
	        throw new FinderException("Account " + acctId + " does not exist.");
	    } catch (Exception r) {
	        throw new EJBException();
	    }
	}

	/**
	 * Transfers the specified amount from the fromAcctId account to the toAcctId account.
	 * @param fromAcctId long The account Id of the account from which the amount is transfered.
	 * @param toAcctId long The account Id of the account to which the amount is transfered.
	 * @param amount float The amount being transferred from the fromAcctId to the toAcctId.
	 * @exception java.rmi.RemoteException A server-side exception caused the method to fail.
	 * @exception javax.ejb.FinderException The specified account Id was not found.
	 * @exception com.ibm.examples.bank.exception.InsufficientFundsException The account identified by
	 * the toAcctId contained less than the amount requested for transfer.
	 */
	public void transferFunds(int fromAcctId, int toAcctId, float amount) throws EJBException, InsufficientFundsException, FinderException {

	    AccountKey fromKey = new AccountKey(fromAcctId);
	    AccountKey toKey = new AccountKey(toAcctId);
	    AccountLocal fromAccount, toAccount;
	    try {
	        fromAccount = accountHome.findByPrimaryKey(fromKey);
	    } catch (ObjectNotFoundException ex) {
	        throw new FinderException("Account " + fromAcctId + " does not exist.");
	    } catch (FinderException ex) {
	        throw new FinderException("Account " + fromAcctId + " does not exist.");
	    } catch (Exception r) {
	        throw new EJBException();
	    }

	    try {
	        toAccount = accountHome.findByPrimaryKey(toKey);
	    } catch (ObjectNotFoundException ex) {
	        throw new FinderException("Account " + toAcctId + " does not exist.");
	    } catch (FinderException ex) {
	        throw new FinderException("Account " + toAcctId + " does not exist.");
	    } catch (Exception r) {
	        throw new EJBException();
	    }

	    try {
	        toAccount.add(amount);
	        fromAccount.subtract(amount);
	    } catch (InsufficientFundsException ex) {
	        mySessionCtx.setRollbackOnly();
	        throw new InsufficientFundsException("Insufficient fund in " + fromAcctId);
	    } catch (Exception r) {
	        throw new EJBException();
	    }
	}

	/**
	 * Gets the home object for the Container Managed Account EJB using ejb references
	 * @exception java.rmi.RemoteException A server-side exception caused the method to fail.
	 */
	private AccountLocalHome getAccountHome() throws RemoteException {
	    try {
	        InitialContext initCtx = new InitialContext();
	        Object objref = initCtx.lookup(ACCOUNT_JNDI_NAME);

	        return(AccountLocalHome) objref;
	    } catch (NamingException ne) {
	        ne.printStackTrace();
	        throw new RemoteException("Error looking up AccountHome object: " + ne.getMessage());
	    }
	}

	/**
	 * ejbCreate Notifies the bean instance that it is being created.
	 */
	public void ejbCreate() {
	    try {
	        accountHome = getAccountHome();
	    } catch (Exception e) {
	        e.printStackTrace();
	        throw new EJBException("Error getting accountHome: " + e.getMessage());
	    }
	}


	/**
	 * getSessionContext
	 */
	public javax.ejb.SessionContext getSessionContext() {
		return mySessionCtx;
	}
	/**
	 * setSessionContext
	 */
	public void setSessionContext(javax.ejb.SessionContext ctx) {
		mySessionCtx = ctx;
	}


	/**
	 * ejbActivate
	 */
	public void ejbActivate() {
	}
	/**
	 * ejbPassivate
	 */
	public void ejbPassivate() {
	}
	/**
	 * ejbRemove
	 */
	public void ejbRemove() {
	}
}